pub struct ClickTracker { /* private fields */ }Expand description
Implementations
Sourceimpl ClickTracker
impl ClickTracker
Sourcepub fn track<V: IntoView>(&self, view: V) -> impl IntoView + use<V>
pub fn track<V: IntoView>(&self, view: V) -> impl IntoView + use<V>
Wrap a view to track when it receives clicks.
The view will have on_click_stop added to it.
Sourcepub fn track_named<V: IntoView>(
&self,
name: &str,
view: V,
) -> impl IntoView + use<V>
pub fn track_named<V: IntoView>( &self, name: &str, view: V, ) -> impl IntoView + use<V>
Wrap a view to track when it receives clicks, with a name for identification.
The view will have on_click_stop added to it.
Sourcepub fn track_named_cont<V: IntoView>(
&self,
name: &str,
view: V,
) -> impl IntoView + use<V>
pub fn track_named_cont<V: IntoView>( &self, name: &str, view: V, ) -> impl IntoView + use<V>
Wrap a view to track when it receives clicks, with a name for identification.
The view will have on_click_cont added to it, allowing the event to bubble.
Sourcepub fn was_clicked(&self) -> bool
pub fn was_clicked(&self) -> bool
Returns true if any tracked view was clicked.
Sourcepub fn click_count(&self) -> usize
pub fn click_count(&self) -> usize
Returns the number of clicks recorded.
Sourcepub fn clicked(&self) -> Vec<Option<String>>
pub fn clicked(&self) -> Vec<Option<String>>
Returns the names of clicked views in order.
Views tracked without names will be represented as None.
Sourcepub fn clicked_names(&self) -> Vec<String>
pub fn clicked_names(&self) -> Vec<String>
Returns just the names of clicked views (ignoring unnamed ones).
Sourcepub fn track_double_click<V: IntoView>(
&self,
name: &str,
view: V,
) -> impl IntoView + use<V>
pub fn track_double_click<V: IntoView>( &self, name: &str, view: V, ) -> impl IntoView + use<V>
Wrap a view to track when it receives double clicks, with a name for identification.
The view will have on_double_click_stop added to it.
Sourcepub fn double_click_count(&self) -> usize
pub fn double_click_count(&self) -> usize
Returns the number of double clicks recorded.
Sourcepub fn double_clicked_names(&self) -> Vec<String>
pub fn double_clicked_names(&self) -> Vec<String>
Returns the names of double-clicked views in order.
Sourcepub fn track_secondary_click<V: IntoView>(
&self,
name: &str,
view: V,
) -> impl IntoView + use<V>
pub fn track_secondary_click<V: IntoView>( &self, name: &str, view: V, ) -> impl IntoView + use<V>
Wrap a view to track when it receives secondary (right) clicks, with a name.
The view will have on_secondary_click_stop added to it.
Sourcepub fn secondary_click_count(&self) -> usize
pub fn secondary_click_count(&self) -> usize
Returns the number of secondary (right) clicks recorded.
Sourcepub fn secondary_clicked_names(&self) -> Vec<String>
pub fn secondary_clicked_names(&self) -> Vec<String>
Returns the names of secondary-clicked views in order.