pub struct PointerCaptureTracker { /* private fields */ }Expand description
Tracks pointer capture events on views for testing.
This helper makes it easy to verify which views received GainedPointerCapture and LostPointerCapture events.
Example
ⓘ
let tracker = PointerCaptureTracker::new();
let view = tracker.track("my_view", my_view);
// ... set pointer capture ...
assert!(tracker.got_capture_count() > 0);Implementations
Sourceimpl PointerCaptureTracker
impl PointerCaptureTracker
Sourcepub fn track<V: IntoView>(&self, name: &str, view: V) -> impl IntoView + use<V>
pub fn track<V: IntoView>(&self, name: &str, view: V) -> impl IntoView + use<V>
Wrap a view to track pointer capture events with a name.
Sourcepub fn gained_capture_count(&self) -> usize
pub fn gained_capture_count(&self) -> usize
Returns the number of GainedPointerCapture events recorded.
Sourcepub fn lost_capture_count(&self) -> usize
pub fn lost_capture_count(&self) -> usize
Returns the number of LostPointerCapture events recorded.
Sourcepub fn got_capture_names(&self) -> Vec<String>
pub fn got_capture_names(&self) -> Vec<String>
Returns the names of views that got pointer capture, in order.
Sourcepub fn lost_capture_names(&self) -> Vec<String>
pub fn lost_capture_names(&self) -> Vec<String>
Returns the names of views that lost pointer capture, in order.
Sourcepub fn pointer_down_names(&self) -> Vec<String>
pub fn pointer_down_names(&self) -> Vec<String>
Returns the names of views that received PointerDown events, in order.
Sourcepub fn pointer_move_names(&self) -> Vec<String>
pub fn pointer_move_names(&self) -> Vec<String>
Returns the names of views that received PointerMove events, in order.
Sourcepub fn pointer_up_names(&self) -> Vec<String>
pub fn pointer_up_names(&self) -> Vec<String>
Returns the names of views that received PointerUp events, in order.