pub struct ScrollTracker { /* private fields */ }Implementations
Sourceimpl ScrollTracker
impl ScrollTracker
Sourcepub fn track(&self, scroll: Scroll) -> Scroll
pub fn track(&self, scroll: Scroll) -> Scroll
Wrap a Scroll view to track its viewport changes.
Sourcepub fn has_scrolled(&self) -> bool
pub fn has_scrolled(&self) -> bool
Returns true if any scroll events have been recorded.
Sourcepub fn scroll_count(&self) -> usize
pub fn scroll_count(&self) -> usize
Returns the number of scroll events recorded.
Sourcepub fn last_offset(&self) -> Option<Vec2>
pub fn last_offset(&self) -> Option<Vec2>
Returns the last recorded viewport, if any.
Sourcepub fn scroll_position(&self) -> Option<Point>
pub fn scroll_position(&self) -> Option<Point>
Returns the current scroll position (top-left of viewport).